import { select, scaleBand, scaleLinear, max, axisLeft, axisBottom, stack } from 'd3';
import { viz } from './viz'; 
import { data } from '@ayush-shinde/cardio_data'; 

export const main = (container) => {
  const width = container.clientWidth;
  const height = container.clientHeight;
  const margin = { top: 20, right: 20, bottom: 60, left: 60 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  //AGE CATEGORIZATION


  const genderColors = {
    Male: 'steelblue',
    Female: 'salmon'
  };

  const hoverColors = {
    Male: 'darkblue',
    Female: 'darkred'
  };
  
const ageRanges = ['18-25', '26-30', '31-35', '36-40', '41-45', '46-50'];

  const ageData = data.reduce((acc, row) => {
  let ageRange;
  if (row.Age >= 18 && row.Age <= 25) {
    ageRange = '18-25';
  } else if (row.Age <= 30) {
    ageRange = '26-30';
  } else if (row.Age <= 35) {
    ageRange = '31-35';
  } else if (row.Age <= 40) {
    ageRange = '36-40';
  } else if (row.Age <= 45) {
    ageRange = '41-45';
  } else {
    ageRange = '46-50';
  }
  if (!acc[ageRange]) {
    acc[ageRange] = { Male: { totalFitness: 0, count: 0 }, Female: { totalFitness: 0, count: 0 } };
  }
  const genderKey = row.Gender === 'Male' ? 'Male' : 'Female';
  acc[ageRange][genderKey].totalFitness += row.Fitness;
  acc[ageRange][genderKey].count += 1;
  return acc;
}, {});

 

//BAR CHART
const preparedData = ageRanges.map(ageRange => {
  const rangeData = ageData[ageRange] || { Male: { totalFitness: 0, count: 1 }, Female: { totalFitness: 0, count: 1 } };
  return {
    AgeRange: ageRange,
    Male: rangeData.Male.count > 0 ? rangeData.Male.totalFitness / rangeData.Male.count : 0,
    Female: rangeData.Female.count > 0 ? rangeData.Female.totalFitness / rangeData.Female.count : 0,
  };
});

//SCALES
  const xScale = scaleBand()
    .domain(ageRanges)
    .range([0, innerWidth])
    .padding(0.2);

  const yScale = scaleLinear()
    .domain([0, max(preparedData, d => max([d.Male, d.Female]))])
    .nice()
    .range([innerHeight, 0]);

  const genderScale = scaleBand()
    .domain(['Male', 'Female'])
    .rangeRound([0, xScale.bandwidth()])
    .padding(0.05);

  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);

  // X-axis
  g.append('g')
    .attr('transform', `translate(0,${innerHeight})`)
    .call(axisBottom(xScale))
    .append('text')
    .attr('fill', 'black')
    .attr('y', margin.bottom - 10)
    .attr('x', innerWidth / 2)
    .attr('text-anchor', 'middle')
    .text('Age Groups');

  // Y-axis
  g.append('g')
    .call(axisLeft(yScale))
    .append('text')
    .attr('fill', 'black')
    .attr('transform', 'rotate(-90)')
    .attr('y', -margin.left + 20)
    .attr('x', -innerHeight / 2)
    .attr('text-anchor', 'middle')
    .attr('dy', '1em')
    .text('Average Fitness Level');

  // Bars
  ageRanges.forEach(ageRange => {
    const ageGroup = g.append('g')
      .attr('transform', `translate(${xScale(ageRange)}, 0)`);

    ['Male', 'Female'].forEach(gender => {
      ageGroup.append('rect')
        .attr('x', genderScale(gender))
        .attr('y', d => yScale(preparedData.find(d => d.AgeRange === ageRange)[gender]))
        .attr('width', genderScale.bandwidth())
        .attr('height', d => innerHeight - yScale(preparedData.find(d => d.AgeRange === ageRange)[gender]))
        .attr('fill', genderColors[gender])
        .on('mouseover', function(event, d) {
          select(this).attr('fill', hoverColors[gender]);
          tooltip.transition()
            .duration(200)
            .style('opacity', .9);
          tooltip.html(`Gender: ${gender}<br/>Average Fitness Level: ${d[gender]}`)
            .style('left', (event.pageX) + 'px')
            .style('top', (event.pageY - 28) + 'px');
        })
        .on('mouseout', function(event, d) {
          select(this).attr('fill', genderColors[gender]);
          tooltip.transition()
            .duration(500)
            .style('opacity', 0);
        });
    });
  });


  // Legend
  const legend = svg.append('g')
    .attr('transform', `translate(${width - margin.right - 100},${margin.top})`);

  const genders = ['Male', 'Female'];
  const colors = ['steelblue', 'salmon']; 

  genders.forEach((gender, index) => {
    const legendRow = legend.append('g')
      .attr('transform', `translate(0, ${index * 20})`);

    legendRow.append('rect')
      .attr('width', 18)
      .attr('height', 18)
      .attr('fill', colors[index]);

    legendRow.append('text')
      .attr('x', 24)
      .attr('y', 9)
      .attr('dy', '0.35em')
      .text(gender);

    legendRow.attr('transform', `translate(20, ${index * 20})`);
  });
};